#pragma once

#include <SFML/Graphics/RectangleShape.hpp>
#include <SFML/Graphics/RenderWindow.hpp>
#include <SFML/Graphics/Transform.hpp>
#include <SFML/Graphics/Transformable.hpp>
#include <SFML/System/Vector2.hpp>
#include <SFML/Window/Event.hpp>

inline sf::Transformable transformFromPosRot(sf::Vector2f position, float rotation) {
    auto result = sf::Transformable();
    result.setPosition(position);
    result.setRotation(rotation);
    return result;
}

class PosTracker {
    /// Wheel encoder values.
    float _wheelDistance, _wheelDiameter;
    float _wheelStepsPerTurn;
    float _lastLeftValue, _lastRightValue;
    sf::Transformable transform;
    public:
        PosTracker(sf::Vector2f initialPos, float initialRot, float wheelDistance, float wheelDiameter, float wheelStepsPerTurn) :
            _wheelDistance(wheelDistance),
            _wheelDiameter(wheelDiameter),
            _wheelStepsPerTurn(wheelStepsPerTurn),
            _lastLeftValue(0.),
            _lastRightValue(0.),
            transform(transformFromPosRot(initialPos, initialRot))
        {}
        void reset(sf::Vector2f initialPos, float initialRot);
        void update(float newLeftValue, float newRightValue);

        inline sf::Vector2f getPosition() const { return transform.getPosition(); }
        inline float getRotation() const { return transform.getRotation(); }
    private:
        inline float wheelIncrementDistance() const {
            return _wheelDiameter * std::numbers::pi / _wheelStepsPerTurn;
        }
        void updateTransform(float leftDistance, float rightDistance);
};
